@@ -3,12 +3,12 @@
 #include "headers.h"
 
 double
-KL_V_xi_e (const gsl_vector *v_V_xi_e)
+KL_V_xi_e (const gsl_vector *v_V_xi_e, Rf_params *params)
 {
   params->V_xi_e[*params->p] = gsl_vector_get(v_V_xi_e, 0);
   double tmpsum = 0.0, tmp;
   double KL;
-  tmpsum = loglikefunc();
+  tmpsum = loglikefunc(params);
   tmp = -pow (params->V_xi_e[*params->p] - *params->xi, 2.0);
   KL = fabs (tmpsum + 0.5*(tmp) / *params->psi2);
   //KL = fabs (tmpsum + (tmp) / *params->psi2);
@@ -16,20 +16,20 @@ KL_V_xi_e (const gsl_vector *v_V_xi_e)
 }
 
 double
-KL_V_xi_n (const gsl_vector *v_V_xi_n)
+KL_V_xi_n (const gsl_vector *v_V_xi_n, Rf_params *params)
 {
   int P_n=*params->P_n;
   params->V_xi_n[*params->i* P_n+*params->p] = gsl_vector_get(v_V_xi_n, 0);
   double tmpsum = 0.0, tmp;
   double KL;
-  tmpsum = loglikefunc();
+  tmpsum = loglikefunc(params);
   tmp = -pow (params->V_xi_n[*params->i + *params->N* *params->p] - *params->xi, 2.0);
   KL = fabs (tmpsum + 0.5*(tmp) / *params->psi2);
   //KL = fabs (tmpsum + (tmp) / *params->psi2);
   return KL;
 }
 
-void gr_KL_V_xi_e (const gsl_vector *v_V_xi_e, void *null, gsl_vector *df)
+void gr_KL_V_xi_e (const gsl_vector *v_V_xi_e, void *null, gsl_vector *df, Rf_params *params)
 {
   int i, j, p=*params->p, pn;
   int P_e=*params->P_e, P_n=*params->P_n;
@@ -77,7 +77,7 @@ void gr_KL_V_xi_e (const gsl_vector *v_V_xi_e, void *null, gsl_vector *df)
   gsl_vector_set(df, 0, -KL);
   return;
 }
-void gr_KL_V_xi_n (const gsl_vector *v_V_xi_n, void *null, gsl_vector *df)
+void gr_KL_V_xi_n (const gsl_vector *v_V_xi_n, void *null, gsl_vector *df, Rf_params *params)
 {
   int i=*params->i, p=*params->p, P_e=*params->P_e, j, pe;
   int N = *params->N;
@@ -181,22 +181,22 @@ void gr_KL_V_xi_n (const gsl_vector *v_V_xi_n, void *null, gsl_vector *df)
 }
  void
      xi_e_fdf (const gsl_vector *x, 
-             void *null, double *f, gsl_vector *df)
+             void *null, double *f, gsl_vector *df, Rf_params *params)
      {
-       *f = KL_V_xi_e(x);
-       gr_KL_V_xi_e(x, NULL, df);
+       *f = KL_V_xi_e(x, params);
+       gr_KL_V_xi_e(x, NULL, df, params);
        return;
      }
 void
      xi_n_fdf (const gsl_vector *x, 
-             void *null, double *f, gsl_vector *df)
+             void *null, double *f, gsl_vector *df, Rf_params *params)
      {
-       *f = KL_V_xi_n(x);
-       gr_KL_V_xi_n(x, NULL, df);
+       *f = KL_V_xi_n(x, params);
+       gr_KL_V_xi_n(x, NULL, df, params);
        return;
      }
 
-double KL_V_z_i (const gsl_vector *v_V_z_i)
+double KL_V_z_i (const gsl_vector *v_V_z_i, Rf_params *params)
 {
   int i = *params->i, d = *params->d;
   int D = *params->D;	
@@ -206,7 +206,7 @@ double KL_V_z_i (const gsl_vector *v_V_z_i)
   double KL;
   for (d=0; d<D; d++)
     params->V_z[i*D+d] = gsl_vector_get(v_V_z_i, d);
-  tmpsum = loglikefunc();
+  tmpsum = loglikefunc(params);
   KL = tmpsum;
   tmpsum = 0;
   for (g = 0; g < *params->G; g++)
@@ -224,7 +224,7 @@ double KL_V_z_i (const gsl_vector *v_V_z_i)
   return KL;
 }
 
-void gr_KL_V_z_i (const gsl_vector *v_V_z_i, void *null, gsl_vector *df)
+void gr_KL_V_z_i (const gsl_vector *v_V_z_i, void *null, gsl_vector *df, Rf_params *params)
 {
   int i = *params->i, j, k, d;
   int D = *params->D;     
@@ -342,15 +342,15 @@ void gr_KL_V_z_i (const gsl_vector *v_V_z_i, void *null, gsl_vector *df)
 
  void
      z_i_fdf (const gsl_vector *x, 
-             void *null, double *f, gsl_vector *df)
+             void *null, double *f, gsl_vector *df, Rf_params *params)
      {
-       *f = KL_V_z_i(x);
-       gr_KL_V_z_i(x, NULL, df);
+       *f = KL_V_z_i(x, params);
+       gr_KL_V_z_i(x, NULL, df, params);
        return;
      }
 
 double
-KL_V_sigma2_i (const gsl_vector *v_V_sigma2_i)
+KL_V_sigma2_i (const gsl_vector *v_V_sigma2_i, Rf_params *params)
 {
   int i = *params->i, j;
   int D = *params->D;	
@@ -359,7 +359,7 @@ KL_V_sigma2_i (const gsl_vector *v_V_sigma2_i)
   double KL;
   int N = *params->N;
   params->V_sigma2[*params->i] = gsl_vector_get(v_V_sigma2_i, 0);
-  tmpsum = loglikefunc();
+  tmpsum = loglikefunc(params);
   double tmpsum1 = 0.0;
   for (g = 0; g < G; g++)
     tmpsum1 =
@@ -375,7 +375,7 @@ KL_V_sigma2_i (const gsl_vector *v_V_sigma2_i)
   return KL;
 }
 
-void gr_KL_V_sigma2_i (const gsl_vector *v_V_sigma2_i, void *null, gsl_vector *df)
+void gr_KL_V_sigma2_i (const gsl_vector *v_V_sigma2_i, void *null, gsl_vector *df, Rf_params *params)
 {
   int i = *params->i, j, k, d = *params->d;
   int D = *params->D;     
@@ -476,16 +476,16 @@ void gr_KL_V_sigma2_i (const gsl_vector *v_V_sigma2_i, void *null, gsl_vector *d
 
  void
      sigma2_i_fdf (const gsl_vector *x, 
-             void *null, double *f, gsl_vector *df)
+             void *null, double *f, gsl_vector *df, Rf_params *params)
      {
-       *f = KL_V_sigma2_i(x);
-       gr_KL_V_sigma2_i(x, NULL, df);
+       *f = KL_V_sigma2_i(x, params);
+       gr_KL_V_sigma2_i(x, NULL, df, params);
        return;
      }
 
 
 double
-KL_V_alpha_g (const gsl_vector *v_V_alpha_g)
+KL_V_alpha_g (const gsl_vector *v_V_alpha_g, Rf_params *params)
 {
   int i = *params->i;
   int d;
@@ -515,7 +515,7 @@ KL_V_alpha_g (const gsl_vector *v_V_alpha_g)
   return KL;
 }
 
-void gr_KL_V_alpha_g (const gsl_vector *v_V_alpha_g, void *null, gsl_vector *df)
+void gr_KL_V_alpha_g (const gsl_vector *v_V_alpha_g, void *null, gsl_vector *df, Rf_params *params)
 {
   int i = *params->i;
   int g = *params->g;
@@ -542,16 +542,16 @@ void gr_KL_V_alpha_g (const gsl_vector *v_V_alpha_g, void *null, gsl_vector *df)
 
  void
      alpha_g_fdf (const gsl_vector *x, 
-             void *null, double *f, gsl_vector *df)
+             void *null, double *f, gsl_vector *df, Rf_params *params)
      {
-       *f = KL_V_alpha_g(x);
-       gr_KL_V_alpha_g(x, NULL, df);
+       *f = KL_V_alpha_g(x, params);
+       gr_KL_V_alpha_g(x, NULL, df, params);
        return;
      }
 
 
 double
-KL_V_nu_g (const gsl_vector *v_V_nu_g)
+KL_V_nu_g (const gsl_vector *v_V_nu_g, Rf_params *params)
 {
   int i = *params->i;	
   int g, G = *params->G;
@@ -580,7 +580,7 @@ KL_V_nu_g (const gsl_vector *v_V_nu_g)
   return KL;
 }
 
-void gr_KL_V_nu_g (const gsl_vector *v_V_nu_g, void *null, gsl_vector *df)
+void gr_KL_V_nu_g (const gsl_vector *v_V_nu_g, void *null, gsl_vector *df, Rf_params *params)
 {
   int i = *params->i;	//, j, d = *params->d;
   int g, G = *params->G;
@@ -606,21 +606,21 @@ void gr_KL_V_nu_g (const gsl_vector *v_V_nu_g, void *null, gsl_vector *df)
 
  void
      nu_g_fdf (const gsl_vector *x, 
-             void *null, double *f, gsl_vector *df)
+             void *null, double *f, gsl_vector *df, Rf_params *params)
      {
-       *f = KL_V_nu_g(x);
-       gr_KL_V_nu_g(x, NULL, df);
+       *f = KL_V_nu_g(x, params);
+       gr_KL_V_nu_g(x, NULL, df, params);
        return;
      }
 
 
 double
-KL_V_psi2_e (const gsl_vector *v_V_psi2_e)
+KL_V_psi2_e (const gsl_vector *v_V_psi2_e, Rf_params *params)
 {
   int D = *params->D;
   params->V_psi2_e[*params->p] = gsl_vector_get(v_V_psi2_e, 0);
   double tmpsum = 0.0;
-  tmpsum = loglikefunc();
+  tmpsum = loglikefunc(params);
   double KL = tmpsum +
 	    0.5*( D * (log (params->V_psi2_e[*params->p])-log(*params->psi2)) -
 	    D * params->V_psi2_e[*params->p] / *params->psi2);
@@ -628,12 +628,12 @@ KL_V_psi2_e (const gsl_vector *v_V_psi2_e)
   return KL;
 }
 double
-KL_V_psi2_n (const gsl_vector *v_V_psi2_n)
+KL_V_psi2_n (const gsl_vector *v_V_psi2_n, Rf_params *params)
 {
   int D = *params->D;
   params->V_psi2_n[*params->p] = gsl_vector_get(v_V_psi2_n, 0);
   double tmpsum = 0.0;
-  tmpsum = loglikefunc();
+  tmpsum = loglikefunc(params);
   double KL = tmpsum +
 	    0.5*( D * (log (params->V_psi2_n[*params->p])-log(*params->psi2)) -
 	    D * params->V_psi2_n[*params->p] / *params->psi2);
@@ -641,7 +641,7 @@ KL_V_psi2_n (const gsl_vector *v_V_psi2_n)
   return KL;
 }
 
-void gr_KL_V_psi2_e (const gsl_vector *v_V_psi2_e, void * null, gsl_vector *df)
+void gr_KL_V_psi2_e (const gsl_vector *v_V_psi2_e, void * null, gsl_vector *df, Rf_params *params)
 {
   int j, i = *params->i, p=*params->p, pn;
   int D = *params->D, P_e = *params->P_e, P_n = *params->P_n;
@@ -691,7 +691,7 @@ void gr_KL_V_psi2_e (const gsl_vector *v_V_psi2_e, void * null, gsl_vector *df)
   return;
 }
 
-void gr_KL_V_psi2_n (const gsl_vector *v_V_psi2_n, void * null, gsl_vector *df)
+void gr_KL_V_psi2_n (const gsl_vector *v_V_psi2_n, void * null, gsl_vector *df, Rf_params *params)
 {
   int i, p=*params->p, j, pe;
   int D = *params->D;     
@@ -746,18 +746,18 @@ void gr_KL_V_psi2_n (const gsl_vector *v_V_psi2_n, void * null, gsl_vector *df)
 
  void
      psi2_e_fdf (const gsl_vector *x, 
-             void *null, double *f, gsl_vector *df)
+             void *null, double *f, gsl_vector *df, Rf_params *params)
      {
-       *f = KL_V_psi2_e(x);
-       gr_KL_V_psi2_e(x, NULL, df);
+       *f = KL_V_psi2_e(x, params);
+       gr_KL_V_psi2_e(x, NULL, df, params);
        return;
      }
  void
      psi2_n_fdf (const gsl_vector *x, 
-             void *null, double *f, gsl_vector *df)
+             void *null, double *f, gsl_vector *df, Rf_params *params)
      {
-       *f = KL_V_psi2_n(x);
-       gr_KL_V_psi2_n(x, NULL, df);
+       *f = KL_V_psi2_n(x, params);
+       gr_KL_V_psi2_n(x, NULL, df, params);
        return;
      }
 void KL_total (int *imodel,
@@ -802,53 +802,54 @@ void KL_total (int *imodel,
 { 
   int p, i, g, d, flag=0;
   double tmp;
-  params=calloc(1,sizeof(Rf_params));
-  params->seed=seed;
-  params->p=&p;
-  params->i=&i;
-  params->g=&g;
-  params->d=&d;
-  params->flag=&flag;
-  params->P_n=P_n;
-  params->P_e=P_e;
-  params->D=D;
-  params->N=N;
-  params->NE=NE;
-  params->NM=NM;
-  params->NnonE=NnonE;
-  params->G=G;
-  params->Y=Y;
-  params->E=E; // edges matrix. NE X 2
-  params->nonE=nonE; // non-edges matrix. NM X 2
-  params->M=M; // missing-edges matrix. NM X 2
-  params->numedges=numedges;
-  params->EnonE=EnonE;
-  params->diam=diam; 
-  params->hopslist=hopslist; 
-  params->XX_e=XX_e; // design matrix for edge covariates. 
-  params->V_xi_n=V_xi_n;
-  params->V_xi_e=V_xi_e;
-  params->V_psi2_n=V_psi2_n;
-  params->V_psi2_e=V_psi2_e;
-  params->V_z=V_z;
-  params->V_sigma2=V_sigma2;
-  params->V_eta=V_eta;
-  params->V_lambda=V_lambda;
-  params->V_omega2=V_omega2;
-  params->V_nu=V_nu;
-  params->V_alpha=V_alpha;
-  params->xi=xi;
-  params->psi2=psi2;
-  params->sigma2=sigma2;
-  params->omega2=omega2;
-  params->nu=nu;
-  params->alpha=alpha;
-  params->inv_sigma02=inv_sigma02;
-  params->NC=NC;
-  params->imodel=imodel;
+  Rf_params *tmp_params;
+  tmp_params=calloc(1,sizeof(Rf_params));
+  tmp_params->seed=seed;
+  tmp_params->p=&p;
+  tmp_params->i=&i;
+  tmp_params->g=&g;
+  tmp_params->d=&d;
+  tmp_params->flag=&flag;
+  tmp_params->P_n=P_n;
+  tmp_params->P_e=P_e;
+  tmp_params->D=D;
+  tmp_params->N=N;
+  tmp_params->NE=NE;
+  tmp_params->NM=NM;
+  tmp_params->NnonE=NnonE;
+  tmp_params->G=G;
+  tmp_params->Y=Y;
+  tmp_params->E=E; // edges matrix. NE X 2
+  tmp_params->nonE=nonE; // non-edges matrix. NM X 2
+  tmp_params->M=M; // missing-edges matrix. NM X 2
+  tmp_params->numedges=numedges;
+  tmp_params->EnonE=EnonE;
+  tmp_params->diam=diam; 
+  tmp_params->hopslist=hopslist; 
+  tmp_params->XX_e=XX_e; // design matrix for edge covariates. 
+  tmp_params->V_xi_n=V_xi_n;
+  tmp_params->V_xi_e=V_xi_e;
+  tmp_params->V_psi2_n=V_psi2_n;
+  tmp_params->V_psi2_e=V_psi2_e;
+  tmp_params->V_z=V_z;
+  tmp_params->V_sigma2=V_sigma2;
+  tmp_params->V_eta=V_eta;
+  tmp_params->V_lambda=V_lambda;
+  tmp_params->V_omega2=V_omega2;
+  tmp_params->V_nu=V_nu;
+  tmp_params->V_alpha=V_alpha;
+  tmp_params->xi=xi;
+  tmp_params->psi2=psi2;
+  tmp_params->sigma2=sigma2;
+  tmp_params->omega2=omega2;
+  tmp_params->nu=nu;
+  tmp_params->alpha=alpha;
+  tmp_params->inv_sigma02=inv_sigma02;
+  tmp_params->NC=NC;
+  tmp_params->imodel=imodel;
   flag=0;
   // p1
-  *KL = loglikefunc(); 
+  *KL = loglikefunc(tmp_params); 
   // p2
   for (g=0;g<*G;g++)
     for (i=0;i<*N;i++)
@@ -857,7 +858,7 @@ void KL_total (int *imodel,
       for (d=0; d<*D; d++)
         tmp += pow(V_z[i* *D+d] - V_eta[g * *D + d], 2.0);
       tmp = GSQRT(tmp + V_sigma2[i] + V_omega2[g]);
-      *KL += V_lambda[g* *N+i]*(*D * gsl_sf_psi (0.5 * *inv_sigma02 *V_alpha[g])- 0.5 * *inv_sigma02 * params->V_alpha[g] * tmp);
+      *KL += V_lambda[g* *N+i]*(*D * gsl_sf_psi (0.5 * *inv_sigma02 *V_alpha[g])- 0.5 * *inv_sigma02 * tmp_params->V_alpha[g] * tmp);
       }
   // p3
   tmp=0;
@@ -903,7 +904,6 @@ void KL_total (int *imodel,
   for (g=0;g<*G;g++)
     *KL += lgamma(0.5*V_alpha[g])-lgamma(0.5*alpha[g]) + 0.5*(alpha[g]-V_alpha[g])*(gsl_sf_psi(0.5*V_alpha[g]));
   //*KL = -*KL;
-  free(params);
   return;
 }
 
