@@ -1,7 +1,7 @@
 #include "headers.h"
 #include <math.h>
 
-double logistic_log_like()
+double loglikefunc(Rf_params *params)
   {
   double log_like = 0.0, tmp, cov, cov2;
   int i=*params->i, j, d, p, k;
